# frozen_string_literal: true

module Diff
  module Strategy
    class Base
      attr_reader :diff

      def initialize(diff = '')
        @diff = diff
      end

      def olds
        # return hash object
        # {"library-name" => "old_version", ...}
        raise NotImplementedError, "You must implement #{self.class}##{__method__}"
      end

      def updates
        # return hash object
        # {"library-name" => "updated_version", ...}
        raise NotImplementedError, "You must implement #{self.class}##{__method__}"
      end

      def to_hash
        updates.each_with_object({}) do |(name, version), hash|
          hash[name] = { old: olds[name], updated: version }
        end
      end

      def exists?
        !to_hash.empty?
      end

      def output(format)
        case format
        when :markdown
          to_hash.map do |name, versions|
            to_markdown(name, versions[:old], versions[:updated])
          end.join("\n")
        when :text
          to_hash.map do |name, versions|
            "#{name} #{versions[:old]} => #{versions[:updated]}"
          end.join("\n")
        else
          to_hash
        end
      end

      def to_markdown(name, old, updated)
        "- Updated: `#{name}` #{old} => #{updated}"
      end
    end
  end
end
