# frozen_string_literal: true

require 'rubygems'
require 'gems'

module Diff
  module Strategy
    class Bundler < Base
      UPDATES_LINE = /^\+\s+(?<name>[0-9a-zA-Z\-_]+)\s\((?<version>[0-9\.]+)\)$/
      OLDS_LINE = /^\-\s+(?<name>[0-9a-zA-Z\-_]+)\s\((?<version>[0-9\.]+)\)$/

      def updates
        @updates ||= extract(UPDATES_LINE)
      end

      def olds
        @olds ||= extract(OLDS_LINE)
      end

      def to_markdown(gem_name, old, updated)
        gem = Gems.info(gem_name)
        sleep 0.1 # rate limit
        if gem.empty?
          super
        else
          source_code_uri = gem["source_code_uri"] || gem["homepage_uri"]
          compare_uri = "#{source_code_uri}/compare/v#{old}...v#{updated}"
          [
            "- Updated:",
            "[`#{gem_name}`](#{source_code_uri})",
            "[#{old} => #{updated}](#{compare_uri})"
          ].join(' ')
        end
      end

      private

      def extract(regex)
        diff.split("\n").each_with_object({}) do |line, hash|
          if matched = line.match(regex)
            hash[matched[:name]] = matched[:version]
          end
        end
      end
    end
  end
end
